import java.lang.*;
import java.io.*;
import java.net.*;

public class Connection {
    protected final int DELAY = 5000;
    protected Socket    sock;
    protected InputStream  in;
    protected OutputStream out;

    public           Connection( String host, int port )
                            throws UnknownHostException,
                                   IOException {
        sock = new Socket( host, port, true );
        in = sock.getInputStream();
        out = sock.getOutputStream();
    }

    public int       available()
                            throws IOException {
        return in.available();
    }

    public int       receive( byte[] data )
                            throws IOException {
        int tempchar, i;
        for ( i = 0; i < data.length; i++ ) {
            tempchar = getchar();
            if ( tempchar == -1 ) {
                break;
            }
            data[i] = (byte)tempchar;
        }
        return i;
    }

    public void      send( byte[] data )
                            throws IOException {
        int i, max;
        max = data.length;
        for ( i = 0; i < data.length; i++ ) {
            if ( data[i] == 255 ) {
                out.write( data[i] );
            }
            out.write( data[i] );
        }
    }

    protected int    getchar()
                            throws IOException {
        int option, tempchar;

        // ensure there is a character to read
        if ( in.available() == 0 ) {
            try {
                Thread.sleep( DELAY );
            }
            catch ( InterruptedException e ) {};
            if ( in.available() == 0 ) {
                return -1;
            }
        }

        // read a character
        tempchar = in.read();

        // check for the escape character
        while ( tempchar == 255 ) {

            // read 'escaped' character
            tempchar = in.read();

            // if it's 255, then pass on the 255
            if ( tempchar == 255 ) {
                return tempchar;
            }

            // determine what they are telling us
            switch ( tempchar ) {
                case 253:
                    option = in.read();   // command: DO (option)
                    out.write( '\377' );  // reply: WON'T (option)
                    out.write( '\374' );
                    out.write( option );
                    break;
                case 251:
                    option = in.read();   // command: WILL (option)
                    out.write( '\377' );  // reply: DON'T (option)
                    out.write( '\376' );
                    out.write( option );
                    break;
                case 252:
                case 254:
                    option = in.read();   // command: WON'T / DON'T
                    break;
                default:
                    break;                // unknown command
            }

            // ensure there is another character
            if ( in.available() == 0 ) {
                try {
                    Thread.sleep( DELAY );
                }
                catch ( InterruptedException e ) {};
                if ( in.available() == 0 ) {
                    return -1;
                }
            }

            // read another character
            tempchar = in.read();
        }
        
        return tempchar;
    }
}

