import java.lang.*;
import java.util.*;
import java.io.*;

public class ItemList {

    public final static int SORT_PRIORITY = 0;
    public final static int SORT_ENDDATE = 1;
    public final static int SORT_GROUP = 2;
    public final static int SORT_STARTDATE = 3;
    public final static int SORT_ALPHA = 4;

    public final static String HEADER = "Scooby2beDue To-Do List Manager v1.0";
    public final static String NOFILE = "cat: " + ToDo.FILENAME + 
                                        ": No such file or directory";

    protected Vector list = new Vector();
    protected GroupList groups = new GroupList();
    protected RemoteFile file;

    public int sort1 = SORT_PRIORITY;
    public int sort2 = SORT_ENDDATE;

    public ItemList( RemoteFile tFile )
            throws IOException {
        file = tFile;
        groups.addGroup( GroupList.BLANK );
        readFile();
    }

    public GroupList getGroups() {
        return groups;
    }

    public int getNumItems() {
        return list.size();
    }

    public Item getItem( int index )
            throws ArrayIndexOutOfBoundsException {
        return (Item)list.elementAt( index );
    }

    public void addItem( Item tItem ) {
        list.addElement( tItem );
        sort( sort1, sort2 );
    }

    public void deleteItem( int index )
            throws ArrayIndexOutOfBoundsException {
        list.removeElementAt( index );
    }

    public void readFile() throws IOException {
        String tempStr;
        boolean done = false;
        Item newItem;

        // open the file
        file.openRead();

        // read header
        tempStr = readln( done );
        if ( tempStr.compareTo( HEADER ) != 0 ) {
            if ( tempStr.compareTo( NOFILE ) == 0 ) {
                file.closeRead();
                return;
            }
            else {
                throw new IOException();
            }
        }

        // and blank line
        tempStr = readln( done );
        if (( tempStr.compareTo( "" ) != 0 ) ||
                ( done == true )) {
            throw new IOException();
        }

        // read groups
        tempStr = readln( done );
        while ( tempStr.compareTo( "" ) != 0 ) {
            groups.addGroup( tempStr );
            tempStr = readln( done );
        }

        // read items
        tempStr = readln( done );
        while ( tempStr.compareTo( "" ) != 0 ) {
            newItem = new Item();
            newItem.description = tempStr;

            tempStr = readln( done );
            if (( tempStr.compareTo( "" ) == 0 ) ||
                    ( done == true )) {
                throw new IOException();
            }
            newItem.group = tempStr;
        
            tempStr = readln( done );
            if (( tempStr.compareTo( "" ) == 0 ) ||
                    ( done == true )) {
                throw new IOException();
            }
            newItem.startDate = new Date( tempStr );

            tempStr = readln( done );
            if (( tempStr.compareTo( "" ) == 0 ) ||
                    ( done == true )) {
                throw new IOException();
            }
            newItem.endDate = new Date( tempStr );

            tempStr = readln( done );
            if (( tempStr.compareTo( "" ) == 0 ) ||
                    ( done == true )) {
                throw new IOException();
            }
            try {
                newItem.priority = Integer.parseInt( tempStr );
            }
            catch ( NumberFormatException e ) {
                throw new IOException();
            }

            addItem( newItem );

            tempStr = readln( done ); // absorb blank line
            tempStr = readln( done ); // read first line of next item
        }

        // close the file
        file.closeRead();
    }

    protected String readln( boolean done ) throws IOException {
        int bytesRead;
        byte[] readBuf = new byte[80];
        bytesRead = file.readln( readBuf );
        if ( bytesRead == 0 ) {
            done = false;
            return new String( "" );
        }
        else if ( bytesRead < 0 ) {
            done = true;
            return new String( "" );
        }
        done = false;
        return new String( readBuf, 0, 0, bytesRead );
    }

    public void saveFile() throws IOException {
        int i, max;
        Item currItem;

        // open the file for writing
        file.openWrite();

        // write header
        writeln( HEADER );

        // write a blank
        writeln( "" );

        // write the groups
        max = groups.getNumGroups();
        for ( i = 1; i < max; i++ ) {
            writeln( groups.getGroup( i ));
        }

        // write a blank
        writeln( "" );

        // write the items
        max = list.size();
        for ( i = 0; i < max; i++ ) {
            currItem = (Item)list.elementAt( i );
            writeln( currItem.description );
            writeln( currItem.group );
            writeln( currItem.startDate.toGMTString() );
            writeln( currItem.endDate.toGMTString() );
            writeln( Integer.toString( currItem.priority ));
            writeln( "" );
        }

        // close the file
        file.closeWrite();
    }

    protected void writeln( String data ) throws IOException {
        int max;
        byte[] buf;

        max = data.length();
        buf = new byte[ max + 1 ];
        data.getBytes( 0, max, buf, 0 );
        buf[ max ] = (byte)'\n';

        file.write( buf );
    }

    public void sort ( int tSort1, int tSort2 ) {
        int max;
        
        sort1 = tSort1;
        sort2 = tSort2;
        quickSort( 0, list.size() - 1 );
    }

    protected void quickSort ( int start, int stop ) {
        int i, j;
        Item x = new Item();

        i = start;
        j = stop;
        dup( x, (Item)list.elementAt( start ));
        while ( i < j ) {
            while (( lessEqual( x, (Item)list.elementAt( j ))) && ( i < j )) {
                j = j - 1;
            }
            dup( (Item)list.elementAt( i ), (Item)list.elementAt( j ));
            while (( lessEqual( (Item)list.elementAt( i ), x )) && ( i < j )) {
                i = i + 1;
            }
            dup( (Item)list.elementAt( j ), (Item)list.elementAt( i ));
        }
        dup( (Item)list.elementAt( i ), x );
        if ( start < i - 1 ) {
            quickSort( start, i - 1 );
        }
        if ( i + 1 < stop ) {
            quickSort( i + 1, stop );
        }
    }

    protected void dup ( Item dest, Item src ) {
        dest.description = src.description;
        dest.startDate = src.startDate;
        dest.endDate = src.endDate;
        dest.priority = src.priority;
        dest.group = src.group;
    }

    protected boolean lessEqual ( Item left, Item right ) {
        int comp;
        comp = compare( sort1, left, right );
        if ( comp < 0 ) {
            return true;
        }
        else if ( comp == 0 ) {
            comp = compare( sort2, left, right );
            if ( comp < 0 ) {
                return true;
            }
            else if ( comp == 0 ) {
                comp = compare( SORT_ALPHA, left, right );
                if ( comp <= 0 ) {
                    return true;
                }
            }
        }
        return false;
    }

    protected int compare ( int tSort, Item left, Item right ) {
        int comp;

        switch( tSort ) {
        case SORT_PRIORITY:
            if ( left.priority < right.priority ) {
                return -1;
            }
            if ( left.priority > right.priority ) {
                return 1;
            }
            break;
        case SORT_ENDDATE:
            comp = dateCompare( left.endDate, right.endDate );
            if ( comp < 0 ) {
                return -1;
            }
            if ( comp > 0 ) {
                return 1;
            }
            break;
        case SORT_GROUP:
            comp = left.group.compareTo( right.group );
            if ( comp < 0 ) {
                return -1;
            }
            if ( comp > 0 ) {
                return 1;
            }
            break;
        case SORT_STARTDATE:
            comp = dateCompare( left.startDate, right.startDate );
            if ( comp < 0 ) {
                return -1;
            }
            if ( comp > 0 ) {
                return 1;
            }
            break;
        case SORT_ALPHA:
            comp = left.description.compareTo( right.description );
            if ( comp < 0 ) {
                return -1;
            }
            if ( comp > 0 ) {
                return 1;
            }
            break;
        }
        return 0;
    }

    protected int dateCompare( Date left, Date right ) {
        if ( left.getYear() < right.getYear() ) {
            return -1;
        }
        if ( left.getYear() > right.getYear() ) {
            return 1;
        }
        if ( left.getMonth() < right.getMonth() ) {
            return -1;
        }
        if ( left.getMonth() > right.getMonth() ) {
            return 1;
        }
        if ( left.getDate() < right.getDate() ) {
            return -1;
        }
        if ( left.getDate() > right.getDate() ) {
            return 1;
        }
        return 0;
    }

}

