import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;

public class RemoteFile {
    protected Connection conn;
    protected String     filename;
    protected String     host;
    protected boolean    atEOF;
    protected boolean    reading;
    protected boolean    writing;
    protected Vector     target;
    protected Vector     readBuf;
    protected Character  EOF    = new Character( (char)256 );
    protected Character  FOUND  = new Character( (char)257 );
    protected Character  CR     = new Character( (char)13 );
    protected Character  LF     = new Character( (char)10 );
    protected String     prompt = "%~zyxwvutsrqponmlkjihgfedcba~%";

    public           RemoteFile( String tHost, String file ) {
        host = tHost;
        filename = file;
        reading = false;
        writing = false;
        target  = new Vector();
        readBuf = new Vector( 31 );
    }
    
    public boolean   login( String userName, String password ) {

        try {

            // connect
            conn = new Connection( host, 23 );

            // read until find 'ogin'
            if ( readUntil( "ogin:" ) == false ) {
                return false;
            }

            // send username
            sendStr( userName + "\n" );

            // read until find 'assword'
            if ( readUntil( "assword:" ) == false ) {
                return false;
            }

            // send password
            sendStr( password + "\n" );

            // read until find 'TERM'
            if ( readUntil( "TERM" ) != false ) {

                // send term type (dumb)
                sendStr( "dumb\n" );
            }

            // read until no more seems to be coming
            target = new Vector();
            while ( readChar() != EOF ) {
                // do nothing
            }

            // ensure we are in ksh
            sendStr( "ksh\n" );

            // make sure we can use ^D to end files
            sendStr( "set -o ignoreeof\n");

            // reset prompt
            sendStr( "export PS1=" + prompt + "\n" );

            // read until no more seems to be coming
            target = new Vector();
            while ( readChar() != EOF ) {
                // do nothing
            }

            // if we don't get our requested prompt back, login failed
            sendStr( "\n" );
            if ( readUntil( prompt ) == false ) {
                return false;
            }
            else {
                return true;
            }
        }
        catch ( Exception e ) {
            return false;
        }
    }
    
    public void      openRead()
                        throws IOException {
        // make sure this operation is legal
        if (( reading == true ) || ( writing == true )) {
            throw new IOException();
        }
        reading = true;

        // read until no more seems to be coming
        target = new Vector();
        while ( readChar() != EOF ) {
            // do nothing
        }

        // send the command to read the file
        sendStr( "cat " + filename + "\n" );

        // consume the command line
        readUntil( filename + "\r\n" );

        // make sure we terminate at the end of the file
        target = toVector( prompt );

        // make sure we don't allow reads past end of file
        atEOF = false;
    }
        
    public void      closeRead()
                        throws IOException {
        // make sure this operation is legal
        if (( reading == false ) || ( writing == true )) {
            throw new IOException();
        }
        reading = false;

        // make sure we get another prompt
        sendStr( "\n" );

        // read until prompt (consume rest of file, if any)
        readUntil( prompt );

        // read extra prompt (may return failure)
        readUntil( prompt );
    }
    
    public void      openWrite()
                        throws IOException {
        // make sure this operation is legal
        if (( reading == true ) || ( writing == true )) {
            throw new IOException();
        }
        writing = true;

        // read until no more seems to be coming
        target = new Vector();
        while ( readChar() != EOF ) {
            // do nothing
        }

        // send command to write to the file
        sendStr( "cat - >| " + filename + "\n" );
    }
    
    public void      closeWrite()
                        throws IOException {
        // make sure this operation is legal
        if (( reading == true ) || ( writing == false )) {
            throw new IOException();
        }
        writing = false;

        sendStr( "\n\004" );

        // make sure we get our prompt back
        readUntil( prompt );
    }

    public int       read( byte[] data )
                        throws IOException {
        // make sure this operation is legal
        if (( reading == false ) || ( writing == true )) {
            throw new IOException();
        }

        int max = data.length;
        Character tempCharacter;
        int i;

        if ( atEOF == true ) {
            return 0;
        }
        for ( i = 0; i < max; i++ ) {
            tempCharacter = readChar();
            if (( tempCharacter == EOF ) ||
                ( tempCharacter == FOUND )) {
                atEOF = true;
                return i;
            }
            data[i] = (byte)(tempCharacter.charValue());
        }
        return i;
    }

    public int       readln( byte[] data )
                        throws IOException {
        // make sure this operation is legal
        if (( reading == false ) || ( writing == true )) {
            throw new IOException();
        }

        int max = data.length;
        Character tempCharacter;
        int i;

        if ( atEOF == true ) {
            return 0;
        }
        for ( i = 0; i < max; i++ ) {
            tempCharacter = readChar();
            if (( tempCharacter == EOF ) ||
                ( tempCharacter == FOUND )) {
                atEOF = true;
                return i;
            }
            if ( tempCharacter.charValue() == CR.charValue() ) {
                i--;
                continue;
            }
            if ( tempCharacter.charValue() == LF.charValue() ) {
                return i;
            }
            data[i] = (byte)(tempCharacter.charValue());
        }
        return i;
    }

    public void      write( byte[] data )
                        throws IOException {
        // make sure this operation is legal
        if (( reading == true ) || ( writing == false )) {
            throw new IOException();
        }
        conn.send( data );
    }

    protected void sendStr( String data ) throws IOException {        
        byte[] str = new byte[data.length()];
        data.getBytes( 0, data.length(), str, 0 );
        conn.send( str );
    }
    
    protected boolean readUntil( String newTarget ) throws IOException {
        Character tempCharacter;
        
        target = toVector( newTarget );

        while ( true ) {
            tempCharacter = readChar();
            if ( tempCharacter == FOUND ) {
                return true;
            }
            if ( tempCharacter == EOF ) {
                return false;
            }
        }
    }
    
    protected Character readChar() throws IOException {
        byte[] in = new byte[1];
        Character tempCharacter;
        int tempInt;

        while ( true ) {
            if ( target.size() == 0 ) {
                tempInt = conn.receive( in );
                if ( tempInt == 0 ) {
                    return EOF;
                }
                return new Character( (char)(in[0]) );
            }
            if ( allCompare( readBuf, target ) == true ) {
                readBuf = new Vector( 31 );
                return FOUND;
            }
            if ( startCompare( readBuf, target ) == true ) {
                tempInt = conn.receive( in );
                if ( tempInt == 0 ) {
                    return EOF;
                }
                tempCharacter = new Character( (char)(in[0]) );
                readBuf.addElement( tempCharacter );
            }
            else {
                try {
                    tempCharacter = (Character)readBuf.firstElement();
                    readBuf.removeElementAt( 0 );
                }
                catch ( Exception e ) {    // should never happen
                    System.out.println("RemoteFile::readChar fatal error");
                    tempCharacter = EOF;
                }
                return tempCharacter;
            }
        }
    }

    protected boolean startCompare( Vector first, Vector second ) {
        int max, i;

        max = first.size();
        if ( second.size() < max ) {
            max = second.size();
        }
        try {
            for ( i = 0; i < max; i++ ) {
                if ( ((Character)(first.elementAt(i))).equals((Character)(second.elementAt(i))) ) {
                    continue;
                }
                return false;
            }
        }
        catch ( Exception e ) {     // should never happen
            System.out.println("RemoteFile::startCompare fatal error");
        }
        return true;
    }

    protected boolean allCompare( Vector first, Vector second ) {
        int max, i;

        max = first.size();
        if ( second.size() != max ) {
            return false;
        }
        try {
            for ( i = 0; i < max; i++ ) {
                if ( ((Character)(first.elementAt(i))).equals((Character)(second.elementAt(i))) ) {
                    continue;
                }
                return false;
            }
        }
        catch ( Exception e ) {     // should never happen
            System.out.println("RemoteFile::allCompare fatal error");
        }
        return true;
    }

    protected Vector toVector( String data ) {
        int i, max;
        max = data.length();
        Vector tempVector = new Vector( max + 1 );
        for ( i = 0; i < max; i++ ) {
            tempVector.addElement( new Character( data.charAt( i )));
        }
        return tempVector;
    }
}

