import java.util.*;
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;

public class ToDo extends Applet {

    //Main window

    protected List mainList = new List( 10, false );
    protected Choice primarySort = new Choice();
    protected Choice secondarySort = new Choice();
    protected TextField description = new TextField( 40 );
    protected TextField dueMonth = new TextField( 2 );
    protected TextField dueDate = new TextField( 2 );
    protected TextField dueYear = new TextField( 4 );
    protected TextField startMonth = new TextField( 2 );
    protected TextField startDate = new TextField( 2 );
    protected TextField startYear = new TextField( 4 );
    protected List priority = new List( 2, false );
    protected List groupList = new List( 2, false );
    protected Button saveButton = new Button( "Save" );
    protected Button addButton = new Button( "Add..." );
    protected Button updateButton = new Button( "Update" );
    protected Button deleteButton = new Button( "Delete" );
    protected Button groupsButton = new Button( "Groups..." );
        
    // data variables
    protected RemoteFile file;
    protected GroupList groups;
    protected ItemList items;

    // state variables
    protected int editing;
    protected int yesNoType;
    protected boolean doNothing;
    protected boolean allowEdit;

    // constants
    public static final String FILENAME = ".todolist";

    public static final int DONT_CARE = 0;
    public static final int EDIT_FIX = 1;
    public static final int NOT_EDITING = -1;

    public void init ()  {

        // Spit out blurb
        System.out.println( getAppletInfo() );
        
        // set up standard initial data
        priority.addItem( "Boiling" );
        priority.addItem( "Hot" );
        priority.addItem( "Warm" );
        priority.addItem( "Cold" );
        priority.addItem( "Iced" );
        priority.select( 2 );
        priority.makeVisible( 2 );

        primarySort.addItem( "Priority" );
        primarySort.addItem( "Due Date" );
        primarySort.addItem( "Group" );
        primarySort.addItem( "Start Date" );
        primarySort.select( 0 );

        secondarySort.addItem( "Priority" );
        secondarySort.addItem( "Due Date" );
        secondarySort.addItem( "Group" );
        secondarySort.addItem( "StartDate" );
        secondarySort.select( 1 );

        // initialize states
        editing = NOT_EDITING;
        yesNoType = DONT_CARE;
        doNothing = false;
        allowEdit = false;

        // Draw Screen
        GridBagConstraints gbc;
        GridBagLayout gb;

        // divide the main screen into 4 areas
        //   quad1 - Main list and labels for sorting
        //   quad2 - drop-downs for sorting
        //   quad3 - main information area
        //   quad4 - button column on right
        gb = new GridBagLayout();
        setLayout( gb );
        Panel quad1 = new Panel();
        Panel quad2 = new Panel();
        Panel quad3 = new Panel();
        Panel quad4 = new Panel();
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.8;
        gbc.weighty = 0.5;
        gbc.gridheight = GridBagConstraints.RELATIVE;
        gb.setConstraints( quad1, gbc );
        add( quad1 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.2;
        gbc.weighty = 0.5;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.gridheight = GridBagConstraints.RELATIVE;
        gb.setConstraints( quad2, gbc );
        add( quad2 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.8;
        gbc.weighty = 0.5;
        gb.setConstraints( quad3, gbc );
        add( quad3 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.2;
        gbc.weighty = 0.5;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( quad4, gbc );
        add( quad4 );

        // divide quad1 into into 2 areas
        //   listArea      - Main list area
        //   sortArea - Area for the sort labels
        gb = new GridBagLayout();
        quad1.setLayout( gb );
        Panel listArea      = new Panel();
        Panel sortArea = new Panel();
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.9;
        gb.setConstraints( listArea, gbc );
        quad1.add( listArea );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.1;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( sortArea, gbc );
        quad1.add( sortArea );

        // add spacing on listArea
        listArea.setLayout( new GridLayout( 1, 1, 5, 5 ));

        // divide sortArea into 7 segments
        sortArea.setLayout( new GridLayout( 7, 1, 5, 0 ));
        Panel label4            = new Panel();
        Panel primarySortArea   = new Panel();
        Panel null1             = new Panel();
        Panel label5            = new Panel();
        Panel secondarySortArea = new Panel();
        Panel null2             = new Panel();
        Panel null3             = new Panel();
        sortArea.add( null1 );
        sortArea.add( label4 );
        sortArea.add( primarySortArea );
        sortArea.add( null2 );
        sortArea.add( label5 );
        sortArea.add( secondarySortArea );
        sortArea.add( null3 );

        // add spacing on sort stuff
        label4.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 0 ));
        primarySortArea.setLayout( new FlowLayout( FlowLayout.RIGHT, 0, 0 ));
        label5.setLayout( new FlowLayout( FlowLayout.LEFT, 0, 0 ));
        secondarySortArea.setLayout( new FlowLayout( FlowLayout.RIGHT, 0, 0 ));

        // divide quad 3 into 8 areas
        //   label1         - "Description:"
        //   descArea       - text area for description
        //   label2         - "Due Date:"
        //   duePriArea     - due date, priority label and priority drop-down
        //   label3         - "Start Date:"
        //   startGroupArea - start date, group label and group drop-down
        //   saveArea       - Save Button
        //   nullArea       - nothing goes in this area
        gb = new GridBagLayout();
        quad3.setLayout( gb );
        Panel label1         = new Panel();
        Panel descArea       = new Panel();
        Panel label2         = new Panel();
        Panel duePriArea     = new Panel();
        Panel label3         = new Panel();
        Panel startGroupArea = new Panel();
        Panel saveArea       = new Panel();
        Panel nullArea       = new Panel();
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.11;
        gbc.weighty = 0.25;
        gb.setConstraints( label1, gbc );
        quad3.add( label1 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.89;
        gbc.weighty = 0.25;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( descArea, gbc );
        quad3.add( descArea );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.11;
        gbc.weighty = 0.25;
        gb.setConstraints( label2, gbc );
        quad3.add( label2 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.89;
        gbc.weighty = 0.25;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( duePriArea, gbc );
        quad3.add( duePriArea );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.11;
        gbc.weighty = 0.25;
        gb.setConstraints( label3, gbc );
        quad3.add( label3 );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.89;
        gbc.weighty = 0.25;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( startGroupArea, gbc );
        quad3.add( startGroupArea );
        gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 0.11;
        gbc.weighty = 0.25;
        gb.setConstraints( saveArea, gbc );
        quad3.add( saveArea );
        gbc = new GridBagConstraints();
        gbc.weightx = 0.89;
        gbc.weighty = 0.25;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gb.setConstraints( nullArea, gbc );
        quad3.add( nullArea );

        // add spacing on labels, saveArea and descArea
        label1.setLayout( new FlowLayout( FlowLayout.RIGHT, 5, 5 ));
        label2.setLayout( new FlowLayout( FlowLayout.RIGHT, 5, 5 ));
        label3.setLayout( new FlowLayout( FlowLayout.RIGHT, 5, 5 ));
        saveArea.setLayout( new GridLayout( 1, 1, 5, 5 ));
        descArea.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 5 ));

        // set layout and spacing on duePriArea
        duePriArea.setLayout( new FlowLayout( FlowLayout.LEFT ));
        Panel dueArea      = new Panel();
        Panel priorityArea = new Panel();
        duePriArea.add( dueArea );
        duePriArea.add( priorityArea );

        // set spacing on dueArea
        dueArea.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 0 ));

        // set spacing on priorityArea
        priorityArea.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 0 ));

        // set layout and spacing on startGroupArea
        startGroupArea.setLayout( new FlowLayout( FlowLayout.LEFT ));
        Panel startArea = new Panel();
        Panel groupArea = new Panel();
        startGroupArea.add( startArea );
        startGroupArea.add( groupArea );

        // set spacing on startArea
        startArea.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 0 ));

        // set spacing on groupArea
        groupArea.setLayout( new FlowLayout( FlowLayout.LEFT, 5, 0 ));

        // divide quad4 into 4 segments
        quad4.setLayout( new GridLayout( 4, 1, 5, 5 ));

        // Add main list
        listArea.add( mainList );

        // Add sort labels and drop-downs
        label4.add( new Label( "  Primary Sort:" ));
        primarySortArea.add( primarySort );
        label5.add( new Label( "  Secondary Sort:" ));
        secondarySortArea.add( secondarySort );

        // Add description line
        label1.add( new Label( "Description:" ));
        descArea.add( description );

        // Add due date stuff
        label2.add( new Label( "Due Date:" ));
        dueArea.add( dueMonth );
        dueArea.add( new Label( "/" ));
        dueArea.add( dueDate );
        dueArea.add( new Label( "/" ));
        dueArea.add( dueYear );

        // add priority stuff
        priorityArea.add( new Label( "Priority:" ));
        priorityArea.add( priority );

        // add start date stuff
        label3.add( new Label( "Start Date:" ));
        startArea.add( startMonth );
        startArea.add( new Label( "/" ));
        startArea.add( startDate );
        startArea.add( new Label( "/" ));
        startArea.add( startYear );

        // add group stuff
        groupArea.add( new Label( "Group:" ));
        groupArea.add( groupList );

        // add save button
        saveArea.add( saveButton );

        // add button bar
        quad4.add( addButton );
        quad4.add( updateButton );
        quad4.add( deleteButton );
        quad4.add( groupsButton );

        // log in the user
        URL url = getCodeBase();
        String host = url.getHost();

        file = new RemoteFile( host, FILENAME );
        Frame tFrame = getFrame( this );
        new LoginDlg( this, tFrame, file );
    }

    // re-present login screen if not already logged in
    public void start() {
        if ( doNothing == true ) {
            URL url = getCodeBase();
            String host = url.getHost();

            file = new RemoteFile( host, FILENAME );
            Frame tFrame = getFrame( this );
            new LoginDlg( this, tFrame, file );
        }    
    }

    // Save on stop
    public void stop() {
        try {
            if ( items != null ) {
                items.saveFile();
            }
        }
        catch ( IOException e ) {}
    }

    // Event handler 
    public boolean handleEvent(Event evt) {
        Frame tFrame = null;
        int i, j, max, max2;
        String group1, group2;

        // if the user was unable to log in, disable all functions
        if ( doNothing == true ) {
            return true;
        }

        // always get rid of LOST_FOCUS and GOT_FOCUS events to prevent 
        // problems keeping track of which item is being edited.
        if (( evt.id == Event.LOST_FOCUS ) || ( evt.id == Event.GOT_FOCUS )) {
            return true;
        }

        // ignore input to edit fields when no record selected to edit
        if (( allowEdit == false ) &&
            (( evt.target == description ) ||
            ( evt.target == dueMonth ) ||
            ( evt.target == dueDate ) ||
            ( evt.target == dueYear ) ||
            ( evt.target == startMonth ) ||
            ( evt.target == startDate ) ||
            ( evt.target == startYear ))) {
            return true;
        }
        else if (( allowEdit == false ) && 
            ( evt.target == priority ) && ( evt.id == Event.LIST_SELECT )) { 
            priority.select( 2 );
            priority.makeVisible( 2 );
            return true;
        }
        else if (( allowEdit == false ) && 
            ( evt.target == groupList ) && ( evt.id == Event.LIST_SELECT )) {
            groupList.select( 0 );
            groupList.makeVisible( 0 );
            return true;
        }

        // keep track of whether they are editing the current item -
        // and make sure they don't accidentally lose changes
        if (( evt.target == description ) ||
            ( evt.target == dueMonth ) ||
            ( evt.target == dueDate ) ||
            ( evt.target == dueYear ) ||
            ( evt.target == startMonth ) ||
            ( evt.target == startDate ) ||
            ( evt.target == startYear ) ||
            (( evt.target == priority ) && 
            ( evt.id == Event.LIST_SELECT )) ||
            (( evt.target == groupList ) &&
            ( evt.id == Event.LIST_SELECT ))) {
            if ( mainList.getSelectedIndex() != -1 ) {
                editing = mainList.getSelectedIndex();
            }
        }
        else if ((( evt.target == mainList ) &&
            ( evt.id == Event.LIST_SELECT )) ||
            (( evt.target == primarySort ) &&
            ( evt.id == Event.ACTION_EVENT )) ||
            (( evt.target == secondarySort ) &&
            ( evt.id == Event.ACTION_EVENT )) ||
            ( evt.target == addButton ) ||
            ( evt.target == deleteButton ) ||
            ( evt.target == groupsButton ) ||
            ( evt.target == saveButton )) {
            if ( editing != NOT_EDITING ) {
                yesNoType = EDIT_FIX;
                if (( evt.target == primarySort ) ||
                        ( evt.target == secondarySort )) {
                    primarySort.select( items.sort1 );
                    secondarySort.select( items.sort2 );
                }
                mainList.select( editing );
                mainList.makeVisible( editing );
                tFrame = getFrame( this );
                new YesNoDlg( this, tFrame, 
                    "This item has been changed", "Keep changes?" );
                return true;
            }
        }

        if (( evt.target == primarySort ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            i = primarySort.getSelectedIndex();
            j = secondarySort.getSelectedIndex();
            if ( i == j ) {
                if ( i == 0 ) {
                    j = 1;
                    secondarySort.select( 1 );
                }
                else {
                    secondarySort.select( j - 1 );
                    j--;
                }
            }
            items.sort( i, j );
            showMainList();
            return true;
        }
        else if (( evt.target == secondarySort ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            i = primarySort.getSelectedIndex();
            j = secondarySort.getSelectedIndex();
            if ( i == j ) {
                if ( i == 0 ) {
                    j = 1;
                    secondarySort.select( 1 );
                }
                else {
                    secondarySort.select( j - 1 );
                    j--;
                }
            }
            items.sort( i, j );
            showMainList();
            return true;
        }
        else if (( evt.target == saveButton ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            try {
                items.saveFile();
                tFrame = getFrame( this );
                new MsgDlg( this, tFrame, "To-Do List Saved", "" );
            }
            catch ( IOException e ) {
                tFrame = getFrame( this );
                new MsgDlg( this, tFrame, "IO Error on Save", "" );
            }
            return true;
        }
        else if (( evt.target == addButton ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            tFrame = getFrame( this );
            new NewDlg( this, tFrame, groups );
            return true;
        }
        else if (( evt.target == updateButton ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            if (( editing != NOT_EDITING ) && ( update() == false )) {
                showMainList();
            }
            editing = NOT_EDITING;
            return true;
        }
        else if (( evt.target == deleteButton ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            i = mainList.getSelectedIndex();
            if ( i >= 0 ) {
                items.deleteItem( i );
            }
            showMainList();
            return true;
        }
        else if (( evt.target == groupsButton ) &&
                ( evt.id == Event.ACTION_EVENT )) {
            tFrame = getFrame( this );
            new GroupsDlg( this, tFrame, groups );
            return true;
        }
        else if ( evt.target == mainList ) {
            if ( evt.id == Event.LIST_SELECT ) {
                populate();
            }
            else if ( evt.id == Event.LIST_DESELECT ) {
                depopulate();
            }
            return true;
        }
        else if ( evt.target instanceof YesNoDlg ) {
            if ( yesNoType == DONT_CARE ) {
                // do nothing
            }
            else if ( yesNoType == EDIT_FIX ) {
                yesNoType = DONT_CARE;
                editing = NOT_EDITING;
                if ( ((Boolean)evt.arg ).booleanValue() == true ) {
                    if ( update() == true ) {
                        editing = mainList.getSelectedIndex();
                    }
                    else {
                        showMainList();
                    }
                }
                else {
                    populate();
                }
            }
            return false;
        } 
        else if ( evt.target instanceof GroupsDlg ) {
            showGroupList();
            if ( ( (Boolean)evt.arg ).booleanValue() == true ) {
                max = groups.getNumGroups();
                max2 = items.getNumItems();
                for ( j = 0; j < max2; j++ ) {
                    group1 = ( items.getItem( j )).group;
                    for ( i = 0; i < max; i++ ) {
                        group2 = groups.getGroup( i );
                        if ( group1.compareTo( group2 ) == 0 ) {
                            break;
                        }
                    }
                    if ( i == max ) {
                        items.deleteItem( j );
                        max2--;
                        j--;
                    }
                }
            }
            showMainList();
            return false;
        }
        else if ( evt.target instanceof LoginDlg ) {
            if ( ( (Boolean)evt.arg ).booleanValue() == true ) {
                doNothing = true;
                tFrame = getFrame( this );
                new MsgDlg( this, tFrame, "Login cancelled", 
                    " All functions disabled." );
                return false;
            }
            try {
                items = new ItemList( file );
            }
            catch ( IOException e ) {
                tFrame = getFrame( this );
                new MsgDlg( this, tFrame, "Unable to read file", "" );
                doNothing = true;
                return false;
            }
            groups = items.getGroups();
            showGroupList();
            showMainList();
            return false;
        }
        else if ( evt.target instanceof NewDlg ) {
            items.addItem( (Item)evt.arg );
            showMainList();
            return false;
        }
        else {
            return super.handleEvent( evt );
        }
    }

    // Method to tell them how cool we are
    public String getAppletInfo() {
         return (
             "Java To-Do List Manager\n" +
             "Written by the Scooby2beDue Development Team\n" +
             "CS 3302 Spring 96 - Prof. Abowd\n" +
             "    Jay Pierce \t (Manager)\n" +
             "    Ken Griggs \t (Architect)\n" +
             "    Minh Ho \t (Programmer)\n" +
             "    Tony Saladino \t (Programmer)\n" +
             "    Jon Preston \t (Technical Writer)\n" );
    }

    // Method to get frame for dialogs
    public static Frame getFrame( Component theComponent )  {
        Component currParent = theComponent;
        Frame theFrame = null;

        while (currParent != null) {
            if (currParent instanceof Frame) {
                theFrame = (Frame)currParent;
                break;
            }
            currParent = currParent.getParent();
        }
        return theFrame;
    }

    // fill the edit fields with all of the data about the selected item
    protected void populate() {
        int i, max;
        int target;
        Item data;

        target = mainList.getSelectedIndex();
        if ( target == -1 ) {
            depopulate();
            return;
        }

        // make sure edit fields are accessible
        description.setEditable( true );
        dueMonth.setEditable( true );
        dueDate.setEditable( true );
        dueYear.setEditable( true );
        startMonth.setEditable( true );
        startDate.setEditable( true );
        startYear.setEditable( true );
        allowEdit = true;

        data = items.getItem( target );

        description.setText( data.description );
        if (( data.endDate.getDate() != data.startDate.getDate() ) ||
                ( data.endDate.getMonth() != data.startDate.getMonth() ) ||
                ( data.endDate.getYear() != data.startDate.getYear() )) {
            dueMonth.setText( Integer.toString( data.endDate.getMonth() + 1 ));
            dueDate.setText( Integer.toString( data.endDate.getDate() ));
            dueYear.setText( Integer.toString( data.endDate.getYear() + 1900 ));
        }
        else {
            dueMonth.setText( "" );
            dueDate.setText( "" );
            dueYear.setText( "" );
        }
        startMonth.setText( Integer.toString( data.startDate.getMonth() + 1 ));
        startDate.setText( Integer.toString( data.startDate.getDate() ));
        startYear.setText( Integer.toString( data.startDate.getYear() + 1900 ));
        priority.select( data.priority - 1 );
        priority.makeVisible( data.priority - 1 );
        groupList.select( 0 );
        groupList.makeVisible( 0 );
        max = groupList.countItems();
        for ( i = 0; i < max; i++ ) {
            if ( data.group.compareTo( groupList.getItem( i )) == 0 ) {
                groupList.select( i );
                groupList.makeVisible( i );
                break;                
            }
        }
    }

    // clear edit fields and make them unaccessable
    protected void depopulate() {
        description.setText( "" );
        dueMonth.setText( "" );
        dueDate.setText( "" );
        dueYear.setText( "" );
        startMonth.setText( "" );
        startDate.setText( "" );
        startYear.setText( "" );
        priority.select( 2 );
        priority.makeVisible( 2 );
        groupList.select( 0 );
        groupList.makeVisible( 2 );

        description.setEditable( false );
        dueMonth.setEditable( false );
        dueDate.setEditable( false );
        dueYear.setEditable( false );
        startMonth.setEditable( false );
        startDate.setEditable( false );
        startYear.setEditable( false );
        allowEdit = false;
    }

    protected void showMainList() {
        int i, max;
        String target, current;
        boolean found = false;
    
        target= mainList.getSelectedItem();
        mainList.clear();
        max = items.getNumItems();
        for ( i = 0; i < max; i++ ) {
            current = items.getItem( i ).description;
            mainList.addItem( current );
            if ( target != null ) {
                if ( target.compareTo( current ) == 0 ) {
                    mainList.select( i );
                    mainList.makeVisible( i );
                    found = true;
                }
            }
        }
        if ( max > 0 ) {
            if ( found == false ) {
                mainList.select( 0 );
                mainList.makeVisible( 0 );
            }
            populate();
        }
        else {
            depopulate();
        }
    }

    protected void showGroupList() {
        int i, max;
        
        groupList.clear();
        max = groups.getNumGroups();
        for ( i = 0; i < max; i++ ) {
            groupList.addItem( groups.getGroup( i ));
        }
    }

    protected boolean update() {
        String strName = null;
        Frame  tFrame  = null;
        int newDueMonth, newDueDate, newDueYear;
        int newStartMonth, newStartDate, newStartYear;
        int target;

        target = mainList.getSelectedIndex();
        if ( target == -1 ) {
            return true;
        }

        Item newItem = new Item();
        boolean noDueDate = false;
 
        strName = dueMonth.getText();
        if ( strName.compareTo( "" ) == 0 ) {
            strName = dueDate.getText();
            if ( strName.compareTo( "" ) == 0 ) {
                strName = dueYear.getText();
                if ( strName.compareTo( "" ) == 0 ) {
                    noDueDate = true;
                }
            }
        }
 
        if ( noDueDate == false ) {
            strName = dueMonth.getText();
            if ( strName.compareTo( "" ) == 0 ) {
                error( "Due Date (Month)" );
                return true;
            }
            strName = dueDate.getText();
            if ( strName.compareTo( "" ) == 0 ) {
                error( "Due Date (Day)" );
                return true;
            }
            strName = dueYear.getText();
            if ( strName.compareTo( "" ) == 0 ) {
                error( "Due Date (Year)" );
                return true;
            }
        }
        strName = description.getText();
        if ( strName.compareTo( "" ) == 0 ) {
            error( "Description" );
            return true;
        }
        strName = startMonth.getText();
        if ( strName.compareTo( "" ) == 0 ) {
            error( "Start Date(Month)" );
            return true;
        }
        strName = startDate.getText();
        if ( strName.compareTo( "" ) == 0 ) {
            error( "Start Date(Day)" );
            return true;
        }
        strName = startYear.getText();
        if ( strName.compareTo( "" ) == 0 ) {
            error( "Start Date(Year)" );
            return true;
        }
 
        try {
            if ( noDueDate == false ) {
                newDueMonth = Integer.parseInt( dueMonth.getText() );
                newDueDate = Integer.parseInt( dueDate.getText() );
                newDueYear = Integer.parseInt( dueYear.getText() );
            }
            else {
                newDueMonth = Integer.parseInt( startMonth.getText() );
                newDueDate = Integer.parseInt( startDate.getText() );
                newDueYear = Integer.parseInt( startYear.getText() );
            }
            newStartMonth = Integer.parseInt( startMonth.getText() );
            newStartDate = Integer.parseInt( startDate.getText() );
            newStartYear = Integer.parseInt( startYear.getText() );
        }
        catch ( NumberFormatException e ) {
            badDate( "Date fields must contain integers." );
            return true;
        }
 
        if (( newDueYear < 1900 ) || ( newStartYear < 1900 )) {
            badDate( "Year must be greater than 1900." );
            return true;
        }
        if (( newDueYear > 2037 ) || ( newStartYear > 2037 )) {
            badDate( "Year must be less than 2037." );
            return true;
        }
        if (( newDueMonth < 1 ) || ( newDueMonth > 12 ) ||
            ( newStartMonth < 1 ) || ( newStartMonth > 12 )) {
            badDate( "Month must be between 1 and 12." );
            return true;
        }
        if (( newDueMonth == 2 ) &&
            (((( newDueYear % 4 ) == 0 ) && ( newDueDate > 29 )) ||
            ((( newDueYear % 4 ) != 0 ) && ( newDueDate > 28 )))) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if (( newStartMonth == 2 ) &&
            (((( newStartYear % 4 ) == 0 ) && ( newStartDate > 29 )) ||
            ((( newStartYear % 4 ) != 0 ) && ( newStartDate > 28 )))) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if (( newDueDate < 1 ) || ( newStartDate < 1 )) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if ((( newDueMonth == 1 ) || ( newDueMonth == 3 ) ||
            ( newDueMonth == 5 ) || ( newDueMonth == 7 ) ||
            ( newDueMonth == 8 ) || ( newDueMonth == 10 ) ||
            ( newDueMonth == 12 )) && ( newDueDate > 31 )) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if ((( newDueMonth == 4 ) || ( newDueMonth == 6 ) ||
            ( newDueMonth == 9 ) || ( newDueMonth == 11 )) &&
            ( newDueDate > 30 )) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if ((( newStartMonth == 1 ) || ( newStartMonth == 3 ) ||
            ( newStartMonth == 5 ) || ( newStartMonth == 7 ) ||
            ( newStartMonth == 8 ) || ( newStartMonth == 10 ) ||
            ( newStartMonth == 12 )) && ( newStartDate > 31 )) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
        if ((( newStartMonth == 4 ) || ( newStartMonth == 6 ) ||
            ( newStartMonth == 9 ) || ( newStartMonth == 11 )) &&
            ( newStartDate > 30 )) {
            badDate( "Specified day does not exist in this month." );
            return true;
        }
 
        if ( newDueYear < newStartYear ) {
            badDate( "Due date is prior to start date." );
            return true;
        }
        else if (( newDueYear == newStartYear ) &&
            ( newDueMonth < newStartMonth )) {
            badDate( "Due date is prior to start date." );
            return true;
        }
        else if (( newDueYear == newStartYear ) &&
            ( newDueMonth == newStartMonth ) &&
            ( newDueDate < newStartDate )) {
            badDate( "Due date is prior to start date." );
            return true;
        }
 
        newItem.description = description.getText();
        newItem.startDate = new Date( newStartYear - 1900,
                newStartMonth - 1, newStartDate, 0, 0, 0 );
        if ( noDueDate == true ) {
            newItem.endDate = newItem.startDate;
        }
        else {
            newItem.endDate = new Date( newDueYear - 1900,
                newDueMonth - 1, newDueDate, 0, 0, 0 );
        }
        newItem.group = groupList.getSelectedItem();
        if ( newItem.group == null ) {
            newItem.group = GroupList.BLANK;
        }
        newItem.priority = priority.getSelectedIndex() + 1;

        items.deleteItem( target );
        items.addItem( newItem );
        return false;
    }

    // Message dialog stating empty field.
    protected void error( String field ) {
        Frame tFrame;
 
        tFrame = ToDo.getFrame( this );
        new MsgDlg( this, tFrame, field + " field is empty.",
            "Please fill all text fields" );
    }
 
    // Message dialog stating bad date
    protected void badDate( String msg ) {
        Frame tFrame;
 
        tFrame = ToDo.getFrame( this );
        new MsgDlg( this, tFrame, "Invalid date.",  msg );
    }

}
